----------------------------------------
-- CPU Top Level for Simulation
----------------------------------------

-------------------------------------------------------------------------------
-- Notes
--2007.02.20
---- SRAM̏SRAM̋[HɈړB
--------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;

use IEEE.std_logic_textio.all;
use STD.textio.all;


entity CPU_SIM is

end CPU_SIM;

architecture SIM of CPU_SIM is

    component CPUTOP 
        port(
        CLKIN : in std_logic;   -- clock 50MHz
        LED : out std_logic_vector(7 downto 0);
        -- to SRAM
        SRAM_RW : out std_logic; -- read or write
        SRAM_IO : inout std_logic_vector(31 downto 0);  -- IO data
        SRAM_IOP : inout std_logic_vector(3 downto 0);  -- IO parity
        SRAM_AA : out std_logic_vector(19 downto 0);    -- address
        -- to SRAM(constant)
        SRAM_CLKM0,SRAM_CLKM1 : out std_logic;                      -- SRAM clock
        SRAM_BW : out std_logic_vector(3 downto 0);
        SRAM_CEH1X,SRAM_CEL1X,SRAM_CE2,SRAM_CE2X : out std_logic;
        SRAM_OEX,SRAM_CENX,SRAM_ADVLD : out std_logic;
        SRAM_LBOX : out std_logic;
        SRAM_ZZ : out std_logic;
        -- usb
        USB_RSTX : out std_logic;                        -- USB reset
        USB_RDX : out std_logic;                         -- 0 when read
        USB_WR : out std_logic;                          -- 1 when write
        USB_TXEX : in std_logic;                         -- 1 when buffer is full
        USB_RXFX : in std_logic;                         -- 1 when buffer is empty
        USB_DATA : inout std_logic_vector(7 downto 0)   -- USB data
        );
    end component;

    component IMSRAM 
        port(
        RW : in std_logic; -- read or write
        IO : inout std_logic_vector(31 downto 0);  -- IO data
        IOP : inout std_logic_vector(3 downto 0);  -- IO parity
        AA : in std_logic_vector(19 downto 0);    -- address
        CLKM0,CLKM1 : in std_logic;                      -- SRAM clock
        BW : in std_logic_vector(3 downto 0);
        CEH1X,CEL1X,CE2,CE2X : in std_logic;      -- chip select
        OEX,CENX,ADVLD : in std_logic;
        LBOX : in std_logic;
        ZZ : in std_logic;
        RESET : in std_logic;
        PRE : out std_logic;
        PRED : std_logic_vector(31 downto 0);
        EOF : std_logic
        );
    end component;

    -- SRAM
        -- to SRAM
        signal SRAM_RW : std_logic; -- read or write
        signal SRAM_IO : std_logic_vector(31 downto 0);  -- IO data
        signal SRAM_IOP : std_logic_vector(3 downto 0);  -- IO parity
        signal SRAM_AA : std_logic_vector(19 downto 0);    -- address
        -- to SRAM(constant)
        signal SRAM_CLKM0,SRAM_CLKM1 : std_logic;                      -- SRAM clock
        signal SRAM_BW : std_logic_vector(3 downto 0);
        signal SRAM_CEH1X,SRAM_CEL1X,SRAM_CE2,SRAM_CE2X : std_logic;
        signal SRAM_OEX,SRAM_CENX,SRAM_ADVLD : std_logic;
        signal SRAM_LBOX : std_logic;
        signal SRAM_ZZ : std_logic;

    -- USB
        signal USB_RSTX : std_logic;                        -- USB reset
        signal USB_RDX : std_logic;                         -- 0 when read
        signal USB_WR : std_logic;                          -- 1 when write
        signal USB_TXEX : std_logic;                         -- 1 when buffer is full
        signal USB_RXFX : std_logic;                         -- 1 when buffer is empty
        signal USB_DATA : std_logic_vector(7 downto 0);   -- USB data


    signal PRE : std_logic; -- prepared sram
    signal PREDATA : std_logic_vector(31 downto 0);
    signal RESET : std_logic := '1';
    signal EOF : std_logic;

    signal CLK,PRECLK : std_logic;
    signal LED : std_logic_vector(7 downto 0);

  file IN_F : text is in "ffib.btx";  -- Input file

begin

    CPU_TOP_INST : CPUTOP port map(
        CLKIN =>CLK,
        LED =>LED,
        -- to SRAM
        SRAM_RW => SRAM_RW,
        SRAM_IO => SRAM_IO,
        SRAM_IOP => SRAM_IOP,
        SRAM_AA => SRAM_AA,
        -- to SRAM(constant)
        SRAM_CLKM0 => SRAM_CLKM0,
        SRAM_CLKM1 => SRAM_CLKM1,
        SRAM_BW => SRAM_BW,
        SRAM_CEH1X => SRAM_CEH1X,
        SRAM_CEL1X => SRAM_CEL1X,
        SRAM_CE2 => SRAM_CE2,
        SRAM_CE2X => SRAM_CE2X,
        SRAM_OEX => SRAM_OEX,
        SRAM_CENX => SRAM_CENX,
        SRAM_ADVLD => SRAM_ADVLD,
        SRAM_LBOX => SRAM_LBOX,
        SRAM_ZZ => SRAM_ZZ,
        -- usb
        USB_RSTX => USB_RSTX,
        USB_RDX => USB_RDX,
        USB_WR => USB_WR,
        USB_TXEX => USB_TXEX,
        USB_RXFX => USB_RXFX,
        USB_DATA => USB_DATA
    );

    IMSRAM_INST : IMSRAM port map(
        RW => SRAM_RW,
        IO => SRAM_IO,
        IOP => SRAM_IOP,
        AA => SRAM_AA,
        CLKM0 => SRAM_CLKM0,
        CLKM1 => PRECLK,
        BW => SRAM_BW,
        CEH1X => SRAM_CEH1X,
        CEL1X => SRAM_CEL1X,
        CE2 => SRAM_CE2,
        CE2X => SRAM_CE2X,
        OEX => SRAM_OEX,
        CENX => SRAM_CENX,
        ADVLD => SRAM_ADVLD,
        LBOX => SRAM_LBOX,
        ZZ => SRAM_ZZ,
        RESET => RESET,
        PRE => PRE,
        PRED => PREDATA,
        EOF => EOF
        );

        
        RESET <= '0';

    process
        variable L : line;
        variable V : std_logic_vector(31 downto 0);
    begin
        if PRE = '0' then
            CLK <= '0';
            wait for 20 ns;
            PRECLK <= '0';
            if endfile(IN_F) then
                EOF <= '1';
            else
                EOF <= '0';
                readline(IN_F,L);  --read data
                read(L,V);
                PREDATA <= V;
           end if;
            wait for 20 ns;
            PRECLK <= '1';
            wait until PRECLK = '1';
        else
           PRECLK <= '0';
            wait for 20 ns;
            CLK <= '0';
            wait for 20 ns;
            CLK <= '1';
            wait until CLK = '1';
        end if;
    end process;

end SIM;
