-- Div_4bit_Sub
-- 受け取ったＡ，Ｂの上位4bitについての比較・減算・シフトを行い、Ａ，Ｃを更新する。
-- 例えば　A = "111101101000000000000000"; B = "110000000000000000000000";
-- 　　　　C = "ABCDEFGHIJKLMNOPQRSTUVWXYZab";　のとき、
-- A >= 1.010B なので、
--       A = "011010000000000000000000"; C = "EFGHIJKLMNOPQRSTUVWXYZab1010";
-- を出力する

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Div_4bit_Sub is

	port(IN_A,IN_B : in std_logic_vector(23 downto 0);
	     IN_C      : in std_logic_vector(27 downto 0);
	     RESTin   : in std_logic;
	     REST     : out std_logic;
	     OUT_A    : out std_logic_vector(23 downto 0);
	     OUT_C    : out std_logic_vector(27 downto 0));

end Div_4bit_Sub;

architecture STRUCTURE of Div_4bit_Sub is

signal IN_A28 : std_logic_vector(27 downto 0); -- for compare

signal B15 : std_logic_vector(27 downto 0); -- 1.111B
signal B14 : std_logic_vector(27 downto 0); -- 1.110B
signal B13 : std_logic_vector(27 downto 0); -- 1.101B
signal B12 : std_logic_vector(27 downto 0); -- 1.100B
signal B11 : std_logic_vector(27 downto 0); -- 1.011B
signal B10 : std_logic_vector(27 downto 0); -- 1.010B
signal B9  : std_logic_vector(27 downto 0); -- 1.001B
signal B8  : std_logic_vector(27 downto 0); -- 1.000B
signal B7  : std_logic_vector(27 downto 0); -- 0.111B
signal B6  : std_logic_vector(27 downto 0); -- 0.110B
signal B5  : std_logic_vector(27 downto 0); -- 0.101B
signal B4  : std_logic_vector(27 downto 0); -- 0.100B
signal B3  : std_logic_vector(27 downto 0); -- 0.011B
signal B2  : std_logic_vector(27 downto 0); -- 0.010B
signal B1  : std_logic_vector(27 downto 0); -- 0.001B

signal WinA15 : std_logic; -- A >= 1.111B
signal WinA14 : std_logic; -- A >= 1.110B
signal WinA13 : std_logic; -- A >= 1.101B
signal WinA12 : std_logic; -- A >= 1.100B
signal WinA11 : std_logic; -- A >= 1.011B
signal WinA10 : std_logic; -- A >= 1.010B
signal WinA9  : std_logic; -- A >= 1.001B
signal WinA8  : std_logic; -- A >= 1.000B
signal WinA7  : std_logic; -- A >= 0.111B
signal WinA6  : std_logic; -- A >= 0.110B
signal WinA5  : std_logic; -- A >= 0.101B
signal WinA4  : std_logic; -- A >= 0.100B
signal WinA3  : std_logic; -- A >= 0.011B
signal WinA2  : std_logic; -- A >= 0.010B
signal WinA1  : std_logic; -- A >= 0.001B

signal IN_A28_substracted : std_logic_vector(27 downto 0);

-- signal IN_A28_shifted : std_logic_vector(27 downto 0);

signal C_Data : std_logic_vector(3 downto 0);

begin -- STRUCTURE
   IN_A28 <= RESTin & IN_A & "000";
   --IN_A28 <= '0' & IN_A & "000"; 
   -- 以下、回路面積が大きいが、クリティカルパスは短いと思われる書き方をする
   -- 逆にする場合は B3 <= B1 + B2; などとする(多分)
   B1  <= "0000" & IN_B;                                                 -- 0.001
   B2  <= "000" & IN_B & '0';                                            -- 0.010
   B3  <= ("0000" & IN_B) + ("000" & IN_B & '0');                        -- 0.011
   B4  <= "00" & IN_B & "00";                                            -- 0.100
   B5  <= ("0000" & IN_B) + ("00" & IN_B & "00");                        -- 0.101
   B6  <= ("000" & IN_B & '0') + ("00" & IN_B & "00");                   -- 0.110
   B7  <= ("0000" & IN_B) + ("000" & IN_B & '0') + ("00" & IN_B & "00"); -- 0.111
   B8  <= '0' & IN_B & "000";                                            -- 1.000
   B9  <= ('0' & IN_B & "000") + ("0000" & IN_B);                        -- 1.001
   B10 <= ('0' & IN_B & "000") + ("000" & IN_B & '0');                   -- 1.010
   B11 <= ('0' & IN_B & "000") + ("0000" & IN_B) + ("000" & IN_B & '0'); -- 1.011
   B12 <= ('0' & IN_B & "000") + ("00" & IN_B & "00");                   -- 1.100
   B13 <= ('0' & IN_B & "000") + ("0000" & IN_B) + ("00" & IN_B & "00"); -- 1.101
   B14 <= ('0' & IN_B & "000") + ("000" & IN_B & '0') + ("00" & IN_B & "00");
   B15 <= ('0' & IN_B & "000") + ("0000" & IN_B) + ("000" & IN_B & '0') + ("00" & IN_B & "00");
   
   WinA15 <= '1' when IN_A28 >= B15 else '0';
   WinA14 <= '1' when IN_A28 >= B14 else '0';
   WinA13 <= '1' when IN_A28 >= B13 else '0';
   WinA12 <= '1' when IN_A28 >= B12 else '0';
   WinA11 <= '1' when IN_A28 >= B11 else '0';
   WinA10 <= '1' when IN_A28 >= B10 else '0';
   WinA9  <= '1' when IN_A28 >= B9  else '0';
   WinA8  <= '1' when IN_A28 >= B8  else '0';
   WinA7  <= '1' when IN_A28 >= B7  else '0';
   WinA6  <= '1' when IN_A28 >= B6  else '0';
   WinA5  <= '1' when IN_A28 >= B5  else '0';
   WinA4  <= '1' when IN_A28 >= B4  else '0';
   WinA3  <= '1' when IN_A28 >= B3  else '0';
   WinA2  <= '1' when IN_A28 >= B2  else '0';
   WinA1  <= '1' when IN_A28 >= B1  else '0';
   
   C_Data <= "1111" when WinA15 = '1' else
             "1110" when WinA14 = '1' else
             "1101" when WinA13 = '1' else
             "1100" when WinA12 = '1' else
             "1011" when WinA11 = '1' else
             "1010" when WinA10 = '1' else
             "1001" when WinA9  = '1' else
             "1000" when WinA8  = '1' else
             "0111" when WinA7  = '1' else
             "0110" when WinA6  = '1' else
             "0101" when WinA5  = '1' else
             "0100" when WinA4  = '1' else
             "0011" when WinA3  = '1' else
             "0010" when WinA2  = '1' else
             "0001" when WinA1  = '1' else
             "0000";
   
   IN_A28_substracted <= IN_A28 - B15 when WinA15 = '1' else 
                         IN_A28 - B14 when WinA14 = '1' else 
                         IN_A28 - B13 when WinA13 = '1' else 
                         IN_A28 - B12 when WinA12 = '1' else 
                         IN_A28 - B11 when WinA11 = '1' else 
                         IN_A28 - B10 when WinA10 = '1' else 
                         IN_A28 - B9  when WinA9  = '1' else 
                         IN_A28 - B8  when WinA8  = '1' else 
                         IN_A28 - B7  when WinA7  = '1' else 
                         IN_A28 - B6  when WinA6  = '1' else 
                         IN_A28 - B5  when WinA5  = '1' else 
                         IN_A28 - B4  when WinA4  = '1' else 
                         IN_A28 - B3  when WinA3  = '1' else 
                         IN_A28 - B2  when WinA2  = '1' else 
                         IN_A28 - B1  when WinA1  = '1' else 
                         IN_A28;
                         
   -- IN_A28_shifted <= IN_A28_substracted(23 downto 0) & "0000";                         
                         
   REST <= '1' when IN_A28_substracted(27 downto 23) /= "00000" else
           '0';                      
   OUT_A <= IN_A28_substracted(22 downto 0) & '0';
   OUT_C <= IN_C(23 downto 0) & C_data;      
end STRUCTURE;

