-- FINVSQRTの繰り返し計算部分
-- X(n+1) = (3 - A*X(n)*X(n))*X(n)/2を行う
-- 

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FINVSQRT_repeat_4 is

	port(EXPin    : in std_logic_vector(7 downto 0);
             Zeroin   : in std_logic;
             Ain      : in std_logic_vector(22 downto 0);
	     nextX    : in std_logic_vector(24 downto 0);
	     EXPout   : out std_logic_vector(7 downto 0);
             Zeroout  : out std_logic;
	     Xhout    : out std_logic_vector(12 downto 0);
             Xlout    : out std_logic_vector(11 downto 0);
	     AXout    : out std_logic_vector(29 downto 0));

end FINVSQRT_repeat_4;

architecture STRUCTURE of FINVSQRT_repeat_4 is

signal A    : std_logic_vector(24 downto 0);
signal AX   : std_logic_vector(49 downto 0);         -- A*X(n)
signal Ah   : std_logic_vector(12 downto 0);
signal Al   : std_logic_vector(11 downto 0);
signal Xh   : std_logic_vector(12 downto 0);
signal Xl   : std_logic_vector(11 downto 0);
signal AXhh : std_logic_vector(25 downto 0);
signal AXhl : std_logic_vector(24 downto 0);
signal AXlh : std_logic_vector(24 downto 0);
signal AXll : std_logic_vector(23 downto 0);

begin -- STRUCTURE

A <= '1' & Ain & '0';
Ah <= A(24 downto 12);
Al <= A(11 downto 0);
Xh <= nextX(24 downto 12);
Xl <= nextX(11 downto 0);
AXhh <= Ah * Xh;
AXhl <= Ah * Xl;
AXlh <= Al * Xh;
AXll <= Al * Xl;
AX <= (AXhh & AXll) + ("000000000000" & (('0' & AXhl) + ('0' & AXlh))  & "000000000000");

AXout <= AX(49 downto 20);
Xhout <= Xh;
Xlout <= Xl;
EXPout <= EXPin;
Zeroout <= Zeroin;

end STRUCTURE;
