-- FPU

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FPU is

	port(SRC1,SRC2 : in std_logic_vector(31 downto 0);
	     FUNC      : in std_logic_vector(3 downto 0);
	     CLK       : in std_logic;
        START     : in std_logic;
        FIN       : out std_logic;
	     OUTPUT    : out std_logic_vector(31 downto 0));

end FPU;

architecture BEHAVIOR of FPU is
   component FMUL
       port(SRC1,SRC2 : in std_logic_vector(31 downto 0);
            OUTPUT : out std_logic_vector(31 downto 0));
   end component;
   component FLESS
       port(SRC1,SRC2 : in std_logic_vector(31 downto 0);
            OUTPUT : out std_logic_vector(31 downto 0));
   end component;
   component ITOF
       port(SRC    : in std_logic_vector(31 downto 0);
            OUTPUT : out std_logic_vector(31 downto 0));
   end component;
   component FTOI
       port(SRC    : in std_logic_vector(31 downto 0);
            OUTPUT : out std_logic_vector(31 downto 0));
   end component;
   component FINVSQRT_n
	port(SRC            : in std_logic_vector(31 downto 0);
        CLK            : in std_logic;
        FINVSQRT_START : in std_logic;
        FINVSQRT_END   : out std_logic;
	     OUTPUT         : out std_logic_vector(31 downto 0));
   end component;
   component F_ADSB
       port(SRC1,SRC2   : in std_logic_vector(31 downto 0);
            FUNC        : in std_logic_vector(3 downto 0);
            CLK         : in std_logic;
            FADSB_START : in std_logic;
            FADSB_END   : out std_logic;
            OUTPUT      : out std_logic_vector(31 downto 0));
   end component;
   signal OUT_FMUL        : std_logic_vector(31 downto 0);
   signal OUT_ITOF        : std_logic_vector(31 downto 0);
   signal OUT_FTOI        : std_logic_vector(31 downto 0);
   signal OUT_FLESS       : std_logic_vector(31 downto 0);
   signal OUT_FINVSQRT    : std_logic_vector(31 downto 0);
   signal OUT_F_ADSB      : std_logic_vector(31 downto 0);
   signal FADSB_START     : std_logic;
   signal FADSB_END       : std_logic;
   signal FINVSQRT_START  : std_logic;
   signal FINVSQRT_END    : std_logic;
   signal FUNCb           : std_logic_vector(3 downto 0);
   signal USE1CLK         : std_logic;
   signal HALF            : std_logic;
begin -- BEHAVIOR
   FMUL1     : FMUL       port map (SRC1,SRC2,OUT_FMUL);
   FLESS1    : FLESS      port map (SRC1,SRC2,OUT_FLESS);
   ITOF1     : ITOF       port map (SRC1,OUT_ITOF);
   FTOI1     : FTOI       port map (SRC1,OUT_FTOI);
   FINVSQRT1 : FINVSQRT_n port map (SRC1,CLK,FINVSQRT_START,FINVSQRT_END,OUT_FINVSQRT);
   F_ADSB1   : F_ADSB     port map (SRC1,SRC2,FUNC,CLK,FADSB_START,FADSB_END,OUT_F_ADSB);

----　↓数値計算用
--   OUTPUT <= OUT_F_ADSB   when FUNC(3 downto 1) = "000" else--FUNC = "0000" or FUNC = "0001" else
--             OUT_FMUL     when FUNC = "0010" else
--             OUT_FINVSQRT when FUNC = "0101" else
--             OUT_FLESS    when FUNC = "0110" else
--             OUT_FTOI     when FUNC = "1100" else
--             OUT_ITOF     when FUNC = "1000" else
--             "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";
----　↑数値計算用

--　↓実行用
   OUTPUT <= OUT_FINVSQRT when FINVSQRT_END = '1' else--FUNCb = "0101" else
             OUT_F_ADSB   when FADSB_END = '1' else--FUNC(3 downto 1) = "000" else--FUNC = "0000" or FUNC = "0001" else
             OUT_ITOF     when FUNCb = "1000" else
             OUT_FMUL     when FUNCb = "0010" else
             OUT_FLESS    when FUNCb = "0110" else
             OUT_FTOI     when FUNCb = "1100" else
             "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";
             
   PROC : process(CLK)
   begin
      if CLK'event and CLK = '0' then
         if FUNC(3 downto 1) = "000" and START = '1' then
            FADSB_START <= '1';
         else
            FADSB_START <= '0';
         end if;
         if FUNC = "0101" and START = '1' then
            FINVSQRT_START <= '1';
         else
            FINVSQRT_START <= '0';
         end if;
         FUNCb <= FUNC;
         HALF <= START;
      end if;
      if CLK'event and CLK = '1' then
         if FUNCb(3 downto 1) /= "000" and FUNCb /= "0101" then
            FIN <= HALF;
         else
            FIN <= FINVSQRT_END or FADSB_END;
         end if;
      end if;
   end process PROC;
--　↑実行用
             
             
             
             
             
   --FPU_P : process(SRC1,SRC2,FUNC,CLK)
   --begin -- FPU_P
   --    case FUNC is 
   --       when "0010" => OUTPUT <= OUT_FMUL;
   --       when "0000" => OUTPUT <= OUT_FADD;
   --       when "0001" => OUTPUT <= OUT_FSUB;
   --       when "0110" => OUTPUT <= OUT_FLESS;
   --       when "1000" => OUTPUT <= OUT_ITOF;
   --       when "1100" => OUTPUT <= OUT_FTOI;
   --       when "0011" => OUTPUT <= OUT_FDIV;
   --       when others => OUTPUT <= "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";
   --    end case;
   --end process;
      
end BEHAVIOR;


configuration FPU1 of FPU is
    for BEHAVIOR
    end for;
end FPU1;
